<?php

declare(strict_types=1);

namespace Drupal\book_library;


use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Render\RendererInterface;
use Drupal\book_library\Form\BookFilterForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a list controller for the book entity type.
 */
final class BookListBuilder extends EntityListBuilder
{

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a new BookListBuilder.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   * The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   * The entity storage object.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   * The form builder service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * The request stack service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   * The renderer service.
   */
  public function __construct(
    EntityTypeInterface $entity_type,
    EntityStorageInterface $storage,
    FormBuilderInterface $form_builder = null,
    RequestStack $request_stack = null,
    RendererInterface $renderer = null,
  ) {
    parent::__construct($entity_type, $storage);
    $this->formBuilder = $form_builder ?? \Drupal::service('form_builder');
    $this->requestStack = $request_stack ?? \Drupal::service('request_stack');
    $this->renderer = $renderer ?? \Drupal::service('renderer');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    $entity_type_manager = $container->get('entity_type.manager');
    return new static(
      $entity_type_manager->getDefinition('book'),
      $entity_type_manager->getStorage('book'),
      $container->get('form_builder'), // Get and pass the form_builder service
      $container->get('request_stack'),  // Get and pass the request_stack service
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array
  {
    // $header['id'] = [
    //   'data' => $this->t('ID'),
    //   'field' => 'id',
    //   'sort' => 'desc',
    //   'specifier' => 'id'
    // ];
    // $header['title'] = [
    //   'data' => $this->t('Title'),
    //   'field' => 'field_book_title',
    //   'specifier' => 'field_book_title'
    // ];
    // $header['author'] = [
    //   'data' => $this->t('Author'),
    //   'field' => 'field_book_author',
    //   'specifier' => 'field_book_author'
    // ];
    // $header['publication_year'] = [
    //   'data' => $this->t('Publication Year'),
    //   'field' => 'field_book_publication_year',
    //   'specifier' => 'field_book_publication_year',
    // ];
    $header['genre'] = [
      'data' => $this->t('Genre'),
      'field' => 'field_book_genre',
      'specifier' => 'entity.field_genre_title.value',
    ];
    // $header['status'] = [
    //   'data' => $this->t('Status'),
    //   'field' => 'field_book_status',
    //   'specifier' => 'field_book_status',
    // ];
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array
  {
    /** @var \Drupal\book_library\BookInterface $entity */
    // $row['id'] = $entity->id();
    // $row['field_book_title'] = $entity->get('field_book_title')->value;
    // $row['field_book_author'] = $entity->get('field_book_author')->value;
    // $row['field_book_publication_year'] = $entity->get('field_book_publication_year')->value;
    $genres = [];
    foreach ($entity->get('field_book_genre')->referencedEntities() as $genre) {
      if ($genre->hasField('field_genre_title') && !$genre->get('field_genre_title')->isEmpty()) {
        $genres[] = $genre->get('field_genre_title')->value;
      }
    }
    $row['field_book_genre'] = implode(', ', $genres);

    // $row['field_book_status'] = $entity->get('field_book_status')->value ? $this->t('Available') : $this->t('Checked Out');
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function load(): array
  {
    $entity_query = $this->getStorage()->getQuery();
    $entity_query->accessCheck(TRUE);
    $header = $this->buildHeader();

    $filter_values = $this->requestStack->getCurrentRequest()->query->all();

    // Apply filters to the entity query.
    $this->applyFilters($entity_query, $filter_values);

    $entity_query->pager(50);
    $entity_query->tableSort($header); // Apply sorting.

    $entities = $entity_query->execute();

    return $this->getStorage()->loadMultiple($entities);
  }

  /**
   * Applies the filters to the entity query.
   *
   * @param \Drupal\Core\Entity\Query\Sql\Query $query
   * The entity query to modify.
   * @param array $filters
   * An array of filter values from the query parameters.
   */
  protected function applyFilters(\Drupal\Core\Entity\Query\Sql\Query &$query, array $filters): void
  {
    // Handle title filter.
    if (!empty($filters['query']['title_filter'])) {
      $query->condition('field_book_title', '%' . $filters['query']['title_filter'] . '%', 'LIKE');
    }

    // Handle author filter 
    if (!empty($filters['query']['author_filter'])) {
      $query->condition('field_book_author', '%' . $filters['query']['author_filter'] . '%', 'LIKE');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render(): array
  {
    $parent_render = parent::render();

    // Build the filter form using the injected FormBuilder.
    $exposed_form = $this->formBuilder->getForm(BookFilterForm::class);

    // Add the form to the top of the content.
    $build['exposed_form'] = $exposed_form;
    $build['table'] = $parent_render;
    return $build;
  }
}