<?php

declare(strict_types=1);

namespace Drupal\book_library\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\book_library\BookInterface;

/**
 * Defines the book entity class.
 *
 * @ContentEntityType(
 *   id = "book",
 *   label = @Translation("Book"),
 *   label_collection = @Translation("Books"),
 *   label_singular = @Translation("book"),
 *   label_plural = @Translation("books"),
 *   label_count = @PluralTranslation(
 *     singular = "@count books",
 *     plural = "@count books",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\book_library\BookListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\book_library\Form\BookForm",
 *       "edit" = "Drupal\book_library\Form\BookForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "delete-multiple-confirm" = "Drupal\Core\Entity\Form\DeleteMultipleForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "book",
 *   admin_permission = "administer book",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "field_book_title",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "collection" = "/admin/content/book",
 *     "add-form" = "/admin/content/book/add",
 *     "canonical" = "/admin/content/book/{book}",
 *     "edit-form" = "/admin/content/book/{book}/edit",
 *     "delete-form" = "/admin/content/book/{book}/delete",
 *     "delete-multiple-form" = "/admin/content/book/delete-multiple",
 *   },
 *   field_ui_base_route = "entity.book.settings",
 * )
 */
final class Book extends ContentEntityBase implements BookInterface
{

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array
  {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of the Book entity.'))
      ->setReadOnly(TRUE)
      ->setSetting('unsigned', TRUE);

    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The UUID of the Book entity.'))
      ->setReadOnly(TRUE);

    $fields['field_book_title'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Book Title'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 255,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0,
      ])
      ->setDisplayOptions('view', [
        'type' => 'sting',
        'label' => 'above',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['field_book_author'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Book Author'))
      ->setRequired(TRUE)
      ->setSettings([
        'max_length' => 255,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 3,
      ])
      ->setDisplayOptions('view', [
        'type' => 'string',
        'label' => 'above',
        'weight' => 3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['field_book_publication_year'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Publication Year'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => 4,
      ])
      ->setDisplayOptions('view', [
        'type' => 'number_integer',
        'label' => 'above',
        'weight' => 4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['field_book_description'] = BaseFieldDefinition::create('text_long')
      ->setLabel(t('Book Description'))
      ->setDisplayOptions('form', [
        'type' => 'text_textarea',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'text_default',
        'label' => 'above',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['field_book_genre'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Genres'))
      //    ->setRequired(TRUE)
      ->setSetting('target_type', 'genre')
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => t('Select or create genres'),
        ],
      ])
      ->setSetting('handler', 'default') // Use the default handler.
      ->setSetting('handler_settings', [
        'auto_create' => TRUE, // Enable automatic creation of new genres.
      ])
      ->setDisplayOptions('view', [
        'type' => 'entity_reference_label',
        'label' => 'above',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['field_book_status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Is Book Available?'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Is Book Available?')
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => FALSE,
        ],
        'weight' => 6,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'boolean',
        'label' => 'above',
        'weight' => 6,
        'settings' => [
          'format' => 'enabled-disabled',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }
}
