<?php

namespace Drupal\book_library\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a form similar to Views exposed filters.
 */
class BookFilterForm extends FormBase
{
  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new BookFilterForm.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * The request stack.
   */
  public function __construct(RequestStack $request_stack)
  {
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string
  {
    return 'book_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array
  {
    $form['#attached']['library'][] = 'book_library/book_filter_form';
    $form['#attached']['library'][] = 'core/drupal.autocomplete';
    $form['#prefix'] = '<div class="view-filters">';
    $form['#suffix'] = '</div>';
    $form['#attributes']['class'][] = 'views-exposed-form';
    $form['#attributes']['id'][] = 'views-exposed-form-content-page-1';

    // $form['wrapper'] = [
    //   '#type' => 'container',
    //   '#prefix' => '<div class="views-exposed-form views-exposed-form">',
    //   '#suffix' => '</div>',
    //   '#attributes' => ['class' => ['book-entity-filters form-wrapper js-form-wrapper']],
    // ];

    $form['title_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filter by Title'),
      '#default_value' => $this->getRequestQueryValue('title_filter'),
      '#attributes' => [
        'name' => 'title_filter',
        'placeholder' => $this->t('Enter title name'),
        'id' => 'edit-title-filter',
        'class' => ['views-exposed-form__item', 'form-autocomplete', 'form-text', 'ui-autocomplete-input'],
        'data-autocomplete-path' => '/admin/content/book-title-autocomplete', // Manual override!
      ],
    ];

    $form['author_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filter by Author'),
      '#default_value' => $this->getRequestQueryValue('author_filter'),
      '#attributes' => [
        'name' => 'author_filter',
        'placeholder' => $this->t('Enter author name'),
        'id' => 'edit-author-filter',
        'class' => ['views-exposed-form__item', 'form-autocomplete', 'form-text', 'ui-autocomplete-input'],
        'data-autocomplete-path' => '/admin/content/book-author-autocomplete', // Manual override!
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
      '#prefix' => '<div class="form-actions views-exposed-form__item views-exposed-form__item--actions">',
      '#suffix' => '</div>',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];

    $form['actions']['reset'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reset Filters'),
      '#limit_validation_errors' => [],
      '#submit' => ['::resetFormFilters'],
      '#attributes' => [
        'class' => ['autocomplete-reset-button'],
      ],
    ];

    return $form;
  }

  /**
   * Helper function to get values from the request query.
   *
   * @param string $key
   * The key of the query parameter to retrieve.
   *
   * @return string|null
   * The value of the query parameter, or NULL if not found.
   */
  protected function getRequestQueryValue(string $key): ?string
  {
    $current_request = $this->requestStack->getCurrentRequest();
    return $current_request->query->get($key);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $values = $form_state->getValues();
    $query = [];

    if (!empty($values['title_filter'])) {
      $query['title_filter'] = $values['title_filter'];
    }

    if (!empty($values['author_filter'])) {
      $query['author_filter'] = $values['author_filter'];
    }

    $route_name = 'entity.book.collection';
    $current_request = $this->requestStack->getCurrentRequest();
    $existing_query = $current_request->query->all();
    $new_query = array_merge($existing_query, $query);

    $form_state->setRedirect($route_name, ['query' => $new_query]);
  }

  /**
   * Submit handler to reset the filter form.
   *
   * @param array $form
   * The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * The form state.
   */
  public function resetFormFilters(array &$form, FormStateInterface $form_state)
  {
    $route_name = 'entity.book.collection';
    $form_state->setRedirect($route_name);
  }
}