<?php

declare(strict_types=1);

namespace Drupal\book_library\Plugin\rest\resource;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest\Attribute\RestResource;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Routing\Route;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

#[RestResource(
    id: 'book_resource',
    label: new TranslatableMarkup('Book Resource'),
    uri_paths: [
        'canonical' => '/api/v1/book/{id}',
        'create' => '/api/v1/book',
        'update' => '/api/v1/book/{id}',
        'delete' => '/api/v1/book/{id}',
    ],
)]
final class BookResource extends ResourceBase {

    protected $currentUser;
    protected $entityTypeManager;

    public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        array $serializer_formats,
        LoggerInterface $logger,
        AccountProxyInterface $currentUser,
        EntityTypeManagerInterface $entityTypeManager
    ) {
        parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
        $this->currentUser = $currentUser;
        $this->entityTypeManager = $entityTypeManager;
    }

    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
        return new self(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->getParameter('serializer.formats'),
            $container->get('logger.factory')->get('rest'),
            $container->get('current_user'),
            $container->get('entity_type.manager')
        );
    }

    public function permissions() {
        return [
            'view book entity' => ['title' => 'View Book Entity'],
            'add book entity' => ['title' => 'Add Book Entity'],
            'edit book entity' => ['title' => 'Edit Book Entity'],
            'delete book entity' => ['title' => 'Delete Book Entity'],
        ];
    }

    /**
   * Responds to POST requests.
   *
   * @param mixed $data
   * The data from the request body.
   *
   * @return \Drupal\rest\ResourceResponse
   * The response object.
   */
    public function post($data) {
        try {
            // Validate required fields.
            if (empty($data['title']) || empty($data['author']) || empty($data['publication_year']) || empty($data['description']) || empty($data['genre'])) {
                throw new BadRequestHttpException('Missing required fields.');
            }

            $bookStorage = $this->entityTypeManager->getStorage('book');
            $genreStorage = $this->entityTypeManager->getStorage('genre');

            $genreReferences = [];

            foreach ($data['genre'] as $genreData) {
                if (isset($genreData['target_id']) && $genreData['target_id'] !== null) {
                    // Existing genre.
                    $genreReferences[] = ['target_id' => $genreData['target_id']];
                    } else if (isset($genreData['title'])) {
                    // Create new genre.
                    $newGenre = $genreStorage->create([
                        'field_genre_title' => $genreData['title'],
                    ]);
                    $newGenre->save();
                    $genreReferences[] = ['target_id' => $newGenre->id()];
                } else {
                throw new BadRequestHttpException('Invalid genre data.');
                }
            }

            $book = $bookStorage->create([
                'type' => 'book',
                'field_book_title' => $data['title'],
                'field_book_author' => $data['author'], 
                'field_book_publication_year' => $data['publication_year'], 
                'field_book_description' => $data['description'], 
                'field_book_genre' => $genreReferences, 
            ]);

            $book->save();

            return new ResourceResponse(['message' => 'Book created successfully.', 'id' => $book->id()], 201);
        } catch (\Exception $e) {
            return new ResourceResponse(['error' => $e->getMessage()], 400);
        }
    }

    public function get($id = null): ResourceResponse {
       
            // Load a single book using canonical route.
            $entity = $this->entityTypeManager->getStorage('book')->load($id);
            if (!$entity) {
                throw new NotFoundHttpException((string) new TranslatableMarkup('Book with ID @id not found.', ['@id' => $id]));
            }
            $bookData = [
                'id' => $entity->id(),
                'title' => $entity->label(),
                'author' => $entity->get('field_book_author')->value,
                'publication_year' => $entity->get('field_book_publication_year')->value,
                'description' => $entity->get('field_book_description')->value,
                'genre' => $entity->get('field_book_genre')->value,
            ];
            return new ResourceResponse($bookData);
        
    }

    public function patch($id, array $data): ModifiedResourceResponse {
        if (!$this->currentUser->hasPermission('edit book entity')) {
            return new ModifiedResourceResponse(['error' => 'Permission denied'], 403);
        }

        $entity = $this->entityTypeManager->getStorage('book')->load($id);
        if (!$entity) {
            throw new NotFoundHttpException((string) new TranslatableMarkup('Book with ID @id not found.', ['@id' => $id]));
        }

        $updatedFields = []; // Track updated fields
        $fieldDefinitions = $entity->getFieldDefinitions();

        foreach ($data as $key => $value) {
            if (isset($fieldDefinitions[$key])) {
                $entity->set($key, $value);
                $updatedFields[$key] = $value;
            } else {
                return new ModifiedResourceResponse(['error' => "Invalid field: $key"], 400); // Bad Request
            }
        }

        if (empty($updatedFields)) {
            return new ModifiedResourceResponse(['error' => 'No valid fields provided for update.'], 400); // Bad request
        }

        $entity->save();

        $this->logger->notice('The book resource record @id has been updated.', ['@id' => $id]);

        return new ModifiedResourceResponse($updatedFields, 200); // Return only updated fields
    }


    public function delete($id): ModifiedResourceResponse {
        if (!$this->currentUser->hasPermission('delete book entity')) {
            return new ModifiedResourceResponse(['error' => 'Permission denied'], 403);
        }
        $entity = \Drupal::entityTypeManager()->getStorage('book')->load($id);
        if (!$entity) {
            throw new NotFoundHttpException((string) new TranslatableMarkup('Book with ID @id not found.', ['@id' => $id]));
        }
        $entity->delete();
        $this->logger->notice('The book resource record @id has been deleted.', ['@id' => $id]);
        return new ModifiedResourceResponse(NULL, 204);
    }

    protected function getBaseRoute($canonical_path, $method): Route {
        $route = parent::getBaseRoute($canonical_path, $method);
        if ($method === 'PATCH' || $method === 'DELETE') {
            $route->setRequirement('id', '\d+');
        }
        return $route;
    }
}